from hostingInfo import HostInfo
import sqlite3

from dataclasses import dataclass

@dataclass
class HostStat:
    percentage: float
    count: int
    datacenter: str


class HostStorage:
    def __init__(self, name):
        """
        :param name: Name of the db or path
        """
        self._conn = sqlite3.connect(name)
        self._conn.execute("CREATE table if not EXISTS hosts (domain text PRIMARY KEY, datacenter text)")
        self._conn.commit()

    def getHostInfo(self, domain):
        """
        Obtains the datacener of a given domain

        :param domain: Domain to fetch
        :return: Datacenter
        """
        rows = self._conn.execute("SELECT * FROM hosts WHERE domain=?", (domain,))

        for (domain, datacenter) in rows.fetchall():
            return HostInfo(domain=domain, datacenter=datacenter)

        return None

    def cache(self, hostsInfo):
        """
        Cache the given data

        :param hostInfo: A list of HostInfo to store
        """

        rows = []
        for h in hostsInfo:
            rows.append((h.domain, h.datacenter))

        self._conn.executemany("INSERT OR REPLACE INTO hosts (domain, datacenter) VALUES (?,?)", rows)
        self._conn.commit()

    def stats(self):
        """
        Get stats from the cached values

        :return: HostStat
        """

        rows = self._conn.execute("""
            SELECT datacenter, count(*) AS NumOcc
            FROM hosts
            GROUP BY datacenter
            ORDER BY Numocc asc
            """)

        numTot = self._conn.execute("SELECT COUNT(*) FROM hosts").fetchone()[0]

        result = []
        for (datacenter, numOcc) in rows.fetchall():
            result.append(HostStat(numOcc / numTot, numOcc, datacenter))


        return result




import datetime
import chrome_bookmarks
import argparse
from hostingInfo import HostInfo
from hostStorage import HostStorage

def printInfoUrl(domain, hostStorage, useCache, storeOnCache):
    """
    Print the info for the given url

    :param domain: String of the damain to get the hosting info from
    :param hostStorage: HostStorage where to store the info
    :param useCache: True enable the loading of cached results
    :param storeOnCache: True enable the caching of the result
    """

    # Load info from cache
    hostInfo = None
    if useCache:
        hostInfo = hostStorage.getHostInfo(domain)
        time = "Loaded from cache"
    
    # Get info from the web
    if hostInfo is None:
        start = datetime.datetime.now()
        hostInfo = HostInfo.hostingInfo(domain)
        end = datetime.datetime.now()

        delta = end - start
        time = delta.total_seconds()

        # Store the result
        if storeOnCache:
            hostStorage.cache([hostInfo])

    print(f"{hostInfo.domain:40} {str(hostInfo.datacenter):40} {time:.02F}s")


def urls(args):
    if args.url is None:
        urls = set()
        for url in chrome_bookmarks.urls:
            domain = HostInfo.getDomain(url.url)
            urls.add(domain)
        return list(urls)
    else:
        return [args.url]




if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Check who is hosting websites")
    parser.add_argument('url', type=str, nargs='?',
        help="The URL to check (esample: 'www.google.com')")
    parser.add_argument('--bookmarks', action='store_true', default=True,
        help='Check the host of all bookmarks [default true]')
    parser.add_argument('--stats', action='store_true', default=False,
        help='Print the stats of the used website [default false]')

    group = parser.add_mutually_exclusive_group()
    group.add_argument('--no-cache', action='store_true', default=False,
        help='The cache will not be used to get and store the results [default false]')
    group.add_argument('--force-cache', action='store_true', default=False,
        help='The cached result will not be used [default false]')
    
    args = parser.parse_args()
    
    hostStorage = HostStorage("host_info.sqlite3")

    if args.stats:
        stats = hostStorage.stats()
        for s in stats:
            datacenter = "<Unknown>" if s.datacenter is None else s.datacenter
            print(f"{datacenter:40} {s.count:02} {s.percentage:05.1%}")
    else:
        useCache = not (args.force_cache or args.no_cache)
        storeOnCache = not args.no_cache

        domains = urls(args)
        for d in domains:
            printInfoUrl(d, hostStorage, useCache, storeOnCache)


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="host-checker",
    version="1.0.3",
    description="Simple CLI tool to check who is hosting a website",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/SimoneCorazza/HostChecker",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
)


from pyquery import PyQuery
from urllib.parse import urlparse
import requests

class HostInfo:
    """
    Info about the host
    """

    def __init__(self, datacenter="", domain=""):
        self.datacenter = datacenter
        self.domain = domain
    
    @property
    def datacenter(self):
        return self._datacenter

    @datacenter.setter
    def datacenter(self, value):
        self._datacenter = value

    @property
    def domain(self):
        return self._domain

    @domain.setter
    def domain(self, value):
        self._domain = value

    @staticmethod
    def hostingInfo(url):
        """
        Get the host info about an URL

        :param url: String of the url of whitch retrive the info
        :return: Info about the hosting
        """
        if not url:
            raise ValueError("url is not present")
        
        domain = HostInfo.getDomain(url)

        r = requests.post("https://hostingchecker.com/wp-admin/admin-ajax.php", { 
                "action": "wx__domain_hostcheker",
                "domain": domain,
            })

        hostInfo = HostInfo()

        if r.status_code == requests.codes.ok:
            pq = PyQuery(r.text)
            hostInfo.datacenter = pq(".isp span strong").html() # Get the Datacenter info
            hostInfo.domain = domain

        return hostInfo

    @staticmethod
    def getDomain(url):
        """
        Get the Domain of an URL

        :param url: URL from whitch get the domain
        :return: Domain name
        """
        urlInfo = urlparse(url)

        # Method 'urlparse' populete field 'netloc' or 'path' depending if the url is only the domain or a complete url
        if urlInfo.netloc:
            return urlInfo.netloc
        else:
            return urlInfo.path
        